#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for accounts_daemon.py module."""

import datetime

from google_compute_engine.accounts import accounts_daemon
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class AccountsDaemonTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.mock_watcher = mock.Mock()
    self.mock_utils = mock.Mock()
    self.mock_oslogin = mock.Mock()

    self.mock_setup = mock.create_autospec(accounts_daemon.AccountsDaemon)
    self.mock_setup.logger = self.mock_logger
    self.mock_setup.watcher = self.mock_watcher
    self.mock_setup.utils = self.mock_utils
    self.mock_setup.oslogin = self.mock_oslogin

  @mock.patch('google_compute_engine.accounts.accounts_daemon.accounts_utils')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.logger')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.file_utils')
  def testAccountsDaemon(
      self, mock_lock, mock_logger, mock_watcher, mock_utils):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mocks.attach_mock(mock_utils, 'utils')
    with mock.patch.object(
        accounts_daemon.AccountsDaemon, 'HandleAccounts') as mock_handle:
      accounts_daemon.AccountsDaemon(groups='foo,bar', remove=True, debug=True)
      expected_calls = [
          mock.call.logger.Logger(name=mock.ANY, debug=True, facility=mock.ANY),
          mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
          mock.call.utils.AccountsUtils(
              logger=mock_logger_instance, groups='foo,bar', remove=True,
              gpasswd_add_cmd=mock.ANY, gpasswd_remove_cmd=mock.ANY,
              groupadd_cmd=mock.ANY, useradd_cmd=mock.ANY,
              userdel_cmd=mock.ANY, usermod_cmd=mock.ANY),
          mock.call.lock.LockFile(accounts_daemon.LOCKFILE),
          mock.call.lock.LockFile().__enter__(),
          mock.call.logger.Logger().info(mock.ANY),
          mock.call.watcher.MetadataWatcher().WatchMetadata(
              mock_handle, recursive=True, timeout=mock.ANY),
          mock.call.lock.LockFile().__exit__(None, None, None),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_daemon.accounts_utils')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.metadata_watcher')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.logger')
  @mock.patch('google_compute_engine.accounts.accounts_daemon.file_utils')
  def testAccountsDaemonError(
      self, mock_lock, mock_logger, mock_watcher, mock_utils):
    mock_logger_instance = mock.Mock()
    mock_logger.Logger.return_value = mock_logger_instance
    mocks = mock.Mock()
    mocks.attach_mock(mock_lock, 'lock')
    mocks.attach_mock(mock_logger, 'logger')
    mocks.attach_mock(mock_watcher, 'watcher')
    mocks.attach_mock(mock_utils, 'utils')
    mock_lock.LockFile.side_effect = IOError('Test Error')
    with mock.patch.object(accounts_daemon.AccountsDaemon, 'HandleAccounts'):
      accounts_daemon.AccountsDaemon()
      expected_calls = [
          mock.call.logger.Logger(
              name=mock.ANY, debug=False, facility=mock.ANY),
          mock.call.watcher.MetadataWatcher(logger=mock_logger_instance),
          mock.call.utils.AccountsUtils(
              logger=mock_logger_instance, groups=None, remove=False,
              gpasswd_add_cmd=mock.ANY, gpasswd_remove_cmd=mock.ANY,
              groupadd_cmd=mock.ANY, useradd_cmd=mock.ANY,
              userdel_cmd=mock.ANY, usermod_cmd=mock.ANY),
          mock.call.lock.LockFile(accounts_daemon.LOCKFILE),
          mock.call.logger.Logger().warning('Test Error'),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  def testHasExpired(self):

    def _GetTimestamp(days):
      """Create a timestamp in the correct format with a days offset.

      Args:
        days: int, number of days to add to the current date.

      Returns:
        string, a timestamp with the format '%Y-%m-%dT%H:%M:%S+0000'.
      """
      format_str = '%Y-%m-%dT%H:%M:%S+0000'
      today = datetime.datetime.now()
      timestamp = today + datetime.timedelta(days=days)
      return timestamp.strftime(format_str)

    ssh_keys = {
        None: False,
        '': False,
        'Invalid': False,
        'user:ssh-rsa key user@domain.com': False,
        'user:ssh-rsa key google {"expireOn":"%s"}' % _GetTimestamp(-1): False,
        'user:ssh-rsa key google-ssh': False,
        'user:ssh-rsa key google-ssh {invalid:json}': False,
        'user:ssh-rsa key google-ssh {"userName":"user"}': False,
        'user:ssh-rsa key google-ssh {"expireOn":"invalid"}': False,
        'user:xyz key google-ssh {"expireOn":"%s"}' % _GetTimestamp(1): False,
        'user:xyz key google-ssh {"expireOn":"%s"}' % _GetTimestamp(-1): True,
    }

    for key, expired in ssh_keys.items():
      self.assertEqual(
          accounts_daemon.AccountsDaemon._HasExpired(self.mock_setup, key),
          expired)

  def testParseAccountsData(self):
    user_map = {
        'a': ['1', '2'],
        'b': ['3', '4', '5'],
    }
    accounts_data = 'skip\n'
    for user, keys in user_map.items():
      for key in keys:
        accounts_data += '%s:%s\n' % (user, key)
    # Make the _HasExpired function treat odd numbers as expired SSH keys.
    self.mock_setup._HasExpired.side_effect = lambda key: int(key) % 2 == 0

    self.assertEqual(
        accounts_daemon.AccountsDaemon._ParseAccountsData(
            self.mock_setup, None), {})
    self.assertEqual(
        accounts_daemon.AccountsDaemon._ParseAccountsData(
            self.mock_setup, ''), {})
    expected_users = {'a': ['1'], 'b': ['3', '5']}
    self.assertEqual(accounts_daemon.AccountsDaemon._ParseAccountsData(
        self.mock_setup, accounts_data), expected_users)

  def testParseAccountsDataNonAscii(self):
    accounts_data = [
        'username:rsa ssh-ke%s invalid\n' % chr(165),
        'use%sname:rsa ssh-key\n' % chr(174),
        'username:rsa ssh-key\n',
    ]
    accounts_data = ''.join(accounts_data)
    self.mock_setup._HasExpired.return_value = False
    expected_users = {'username': ['rsa ssh-key']}
    self.assertEqual(accounts_daemon.AccountsDaemon._ParseAccountsData(
        self.mock_setup, accounts_data), expected_users)

  def testGetInstanceAndProjectAttributes(self):

    def _AssertAttributeDict(data, expected):
      """Test the correct accounts data is returned.

      Args:
        data: dictionary, the faux metadata server contents.
        expected: list, the faux SSH keys expected to be set.
      """
      self.assertEqual(
          accounts_daemon.AccountsDaemon._GetInstanceAndProjectAttributes(
              self.mock_setup, data), expected)

    data = None
    _AssertAttributeDict(data, ({}, {}))

    data = {'test': 'data'}
    expected = ({}, {})
    _AssertAttributeDict(data, expected)

    data = {'instance': {'attributes': {}}}
    expected = ({}, {})
    _AssertAttributeDict(data, expected)

    data = {'instance': {'attributes': {'ssh-keys': '1'}}}
    expected = ({'ssh-keys': '1'}, {})
    _AssertAttributeDict(data, expected)

    data = {'instance': {'attributes': {'ssh-keys': '1', 'sshKeys': '2'}}}
    expected = ({'ssh-keys': '1', 'sshKeys': '2'}, {})
    _AssertAttributeDict(data, expected)

    data = {'project': {'attributes': {'ssh-keys': '1'}}}
    expected = ({}, {'ssh-keys': '1'})
    _AssertAttributeDict(data, expected)

    data = {'project': {'attributes': {'ssh-keys': '1', 'sshKeys': '2'}}}
    expected = ({}, {'ssh-keys': '1', 'sshKeys': '2'})
    _AssertAttributeDict(data, expected)

    data = {
        'instance': {
            'attributes': {
                'ssh-keys': '1',
                'sshKeys': '2',
            },
        },
        'project': {
            'attributes': {
                'ssh-keys': '3',
            },
        },
    }
    expected = ({'ssh-keys': '1', 'sshKeys': '2'}, {'ssh-keys': '3'})
    _AssertAttributeDict(data, expected)

    data = {
        'instance': {
            'attributes': {
                'ssh-keys': '1',
                'block-project-ssh-keys': 'false',
            },
        },
        'project': {
            'attributes': {
                'ssh-keys': '2',
            },
        },
    }
    expected = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
                {'ssh-keys': '2'})
    _AssertAttributeDict(data, expected)

    data = {
        'instance': {
            'attributes': {
                'ssh-keys': '1',
                'block-project-ssh-keys': 'true',
            },
        },
        'project': {
            'attributes': {
                'ssh-keys': '2',
            },
        },
    }
    expected = ({'block-project-ssh-keys': 'true', 'ssh-keys': '1'},
                {'ssh-keys': '2'})
    _AssertAttributeDict(data, expected)

    data = {
        'instance': {
            'attributes': {
                'ssh-keys': '1',
                'block-project-ssh-keys': 'false',
            },
        },
        'project': {
            'attributes': {
                'ssh-keys': '2',
                'sshKeys': '3',
            },
        },
    }
    expected = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
                {'sshKeys': '3', 'ssh-keys': '2'})
    _AssertAttributeDict(data, expected)

  def testGetAccountsData(self):

    def _AssertAccountsData(data, expected):
      """Test the correct accounts data is returned.

      Args:
        data: dictionary, the faux metadata server contents.
        expected: list, the faux SSH keys expected to be set.
      """
      self.mock_setup._GetInstanceAndProjectAttributes.return_value = data
      accounts_daemon.AccountsDaemon._GetAccountsData(self.mock_setup, data)
      if expected:
        call_args, _ = self.mock_setup._ParseAccountsData.call_args
        actual = call_args[0]
        self.assertEqual(set(actual.split()), set(expected))
      else:
        self.mock_setup._ParseAccountsData.assert_called_once_with(expected)
      self.mock_setup._ParseAccountsData.reset_mock()

    data = ({}, {})
    _AssertAccountsData(data, '')

    data = ({'ssh-keys': '1'}, {})
    _AssertAccountsData(data, ['1'])

    data = ({'ssh-keys': '1', 'sshKeys': '2'}, {})
    _AssertAccountsData(data, ['1', '2'])

    data = ({}, {'ssh-keys': '1'})
    _AssertAccountsData(data, ['1'])

    data = ({}, {'ssh-keys': '1', 'sshKeys': '2'})
    _AssertAccountsData(data, ['1', '2'])

    data = ({'ssh-keys': '1', 'sshKeys': '2'}, {'ssh-keys': '3'})
    _AssertAccountsData(data, ['1', '2'])

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'ssh-keys': '2'})
    _AssertAccountsData(data, ['1', '2'])

    data = ({'block-project-ssh-keys': 'true', 'ssh-keys': '1'},
            {'ssh-keys': '2'})
    _AssertAccountsData(data, ['1'])

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'sshKeys': '3', 'ssh-keys': '2'})
    _AssertAccountsData(data, ['1', '2', '3'])

  def testGetEnableOsLoginValue(self):

    def _AssertEnableOsLogin(data, expected):
      """Test the correct value for enable-oslogin is returned.

      Args:
        data: dictionary, the faux metadata server contents.
        expected: bool, if True, OS Login is enabled.
      """
      self.mock_setup._GetInstanceAndProjectAttributes.return_value = data
      actual = accounts_daemon.AccountsDaemon._GetEnableOsLoginValue(
          self.mock_setup, data)
      self.assertEqual(actual, expected)

    data = ({}, {})
    _AssertEnableOsLogin(data, False)

    data = ({'enable-oslogin': 'true'}, {})
    _AssertEnableOsLogin(data, True)

    data = ({'enable-oslogin': 'false'}, {})
    _AssertEnableOsLogin(data, False)

    data = ({'enable-oslogin': 'yep'}, {})
    _AssertEnableOsLogin(data, False)

    data = ({'enable-oslogin': 'True'}, {})
    _AssertEnableOsLogin(data, True)

    data = ({'enable-oslogin': 'TRUE'}, {})
    _AssertEnableOsLogin(data, True)

    data = ({'enable-oslogin': ''}, {})
    _AssertEnableOsLogin(data, False)

    data = ({'enable-oslogin': 'true'}, {'enable-oslogin': 'true'})
    _AssertEnableOsLogin(data, True)

    data = ({'enable-oslogin': 'false'}, {'enable-oslogin': 'true'})
    _AssertEnableOsLogin(data, False)

    data = ({'enable-oslogin': ''}, {'enable-oslogin': 'true'})
    _AssertEnableOsLogin(data, True)

    data = ({}, {'enable-oslogin': 'true'})
    _AssertEnableOsLogin(data, True)

    data = ({}, {'enable-oslogin': 'false'})
    _AssertEnableOsLogin(data, False)

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'sshKeys': '3', 'ssh-keys': '2'})
    _AssertEnableOsLogin(data, False)

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'sshKeys': '3', 'ssh-keys': '2', 'enable-oslogin': 'true'})
    _AssertEnableOsLogin(data, True)

  def testGetEnableTwoFactorValue(self):

    def _AssertEnableTwoFactor(data, expected):
      """Test the correct value for enable-oslogin-2fa is returned.

      Args:
        data: dictionary, the faux metadata server contents.
        expected: bool, if True, two factor authentication is enabled.
      """
      self.mock_setup._GetInstanceAndProjectAttributes.return_value = data
      actual = accounts_daemon.AccountsDaemon._GetEnableTwoFactorValue(
          self.mock_setup, data)
      self.assertEqual(actual, expected)

    data = ({}, {})
    _AssertEnableTwoFactor(data, False)

    data = ({'enable-oslogin-2fa': 'true'}, {})
    _AssertEnableTwoFactor(data, True)

    data = ({'enable-oslogin-2fa': 'false'}, {})
    _AssertEnableTwoFactor(data, False)

    data = ({'enable-oslogin-2fa': 'yep'}, {})
    _AssertEnableTwoFactor(data, False)

    data = ({'enable-oslogin-2fa': 'True'}, {})
    _AssertEnableTwoFactor(data, True)

    data = ({'enable-oslogin-2fa': 'TRUE'}, {})
    _AssertEnableTwoFactor(data, True)

    data = ({'enable-oslogin-2fa': ''}, {})
    _AssertEnableTwoFactor(data, False)

    data = ({'enable-oslogin-2fa': 'true'}, {'enable-oslogin-2fa': 'true'})
    _AssertEnableTwoFactor(data, True)

    data = ({'enable-oslogin-2fa': 'false'}, {'enable-oslogin-2fa': 'true'})
    _AssertEnableTwoFactor(data, False)

    data = ({'enable-oslogin-2fa': ''}, {'enable-oslogin-2fa': 'true'})
    _AssertEnableTwoFactor(data, True)

    data = ({}, {'enable-oslogin-2fa': 'true'})
    _AssertEnableTwoFactor(data, True)

    data = ({}, {'enable-oslogin-2fa': 'false'})
    _AssertEnableTwoFactor(data, False)

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'sshKeys': '3', 'ssh-keys': '2'})
    _AssertEnableTwoFactor(data, False)

    data = ({'block-project-ssh-keys': 'false', 'ssh-keys': '1'},
            {'sshKeys': '3', 'ssh-keys': '2', 'enable-oslogin-2fa': 'true'})
    _AssertEnableTwoFactor(data, True)

  def testUpdateUsers(self):
    update_users = {
        'a': '1',
        'b': '2',
        'c': '3',
        'invalid': '4',
        'valid': '5',
        'unchanged': ['1', '2', '3'],
    }
    self.mock_setup.user_ssh_keys = {
        'unchanged': ['3', '2', '1'],
    }
    self.mock_setup.invalid_users = set(['invalid'])
    # Make UpdateUser succeed for fake names longer than one character.
    self.mock_utils.UpdateUser.side_effect = lambda user, _: len(user) > 1
    accounts_daemon.AccountsDaemon._UpdateUsers(self.mock_setup, update_users)
    expected_calls = [
        mock.call('a', '1'),
        mock.call('b', '2'),
        mock.call('c', '3'),
        mock.call('valid', '5'),
    ]
    self.mock_utils.UpdateUser.assert_has_calls(expected_calls, any_order=True)
    self.assertEqual(
        self.mock_utils.UpdateUser.call_count, len(expected_calls))
    self.assertEqual(
        self.mock_setup.invalid_users, set(['invalid', 'a', 'b', 'c']))
    self.assertEqual(
        self.mock_setup.user_ssh_keys,
        {'valid': '5', 'unchanged': ['3', '2', '1']})

  def testRemoveUsers(self):
    remove_users = ['a', 'b', 'c', 'valid']
    self.mock_setup.user_ssh_keys = {
        'a': ['1'],
        'b': ['2'],
        'c': ['3'],
        'invalid': ['key'],
    }
    self.mock_setup.invalid_users = set(['invalid', 'a', 'b', 'c'])
    accounts_daemon.AccountsDaemon._RemoveUsers(self.mock_setup, remove_users)
    expected_calls = [
        mock.call('a'),
        mock.call('b'),
        mock.call('c'),
        mock.call('valid'),
    ]
    self.mock_utils.RemoveUser.assert_has_calls(expected_calls)
    self.assertEqual(self.mock_setup.invalid_users, set(['invalid']))
    self.assertEqual(self.mock_setup.user_ssh_keys, {'invalid': ['key']})

  def testHandleAccountsNoOsLogin(self):
    configured = ['c', 'c', 'b', 'b', 'a', 'a']
    desired = {'d': '1', 'c': '2'}
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_utils, 'utils')
    mocks.attach_mock(self.mock_setup, 'setup')
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_utils.GetConfiguredUsers.return_value = configured
    self.mock_setup._GetAccountsData.return_value = desired
    self.mock_setup._GetEnableOsLoginValue.return_value = False
    self.mock_oslogin.UpdateOsLogin.return_value = 0
    result = 'result'
    expected_add = ['c', 'd']
    expected_remove = ['a', 'b']

    accounts_daemon.AccountsDaemon.HandleAccounts(self.mock_setup, result)
    expected_calls = [
        mock.call.setup.logger.debug(mock.ANY),
        mock.call.utils.GetConfiguredUsers(),
        mock.call.setup._GetEnableOsLoginValue(result),
        mock.call.setup._GetEnableTwoFactorValue(result),
        mock.call.setup._GetAccountsData(result),
        mock.call.oslogin.UpdateOsLogin(False),
        mock.call.setup._UpdateUsers(desired),
        mock.call.setup._RemoveUsers(mock.ANY),
        mock.call.utils.SetConfiguredUsers(mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)
    call_args, _ = self.mock_utils.SetConfiguredUsers.call_args
    self.assertEqual(set(call_args[0]), set(expected_add))
    call_args, _ = self.mock_setup._RemoveUsers.call_args
    self.assertEqual(set(call_args[0]), set(expected_remove))

  def testHandleAccountsOsLogin(self):
    configured = ['c', 'c', 'b', 'b', 'a', 'a']
    desired = {}
    mocks = mock.Mock()
    mocks.attach_mock(self.mock_utils, 'utils')
    mocks.attach_mock(self.mock_setup, 'setup')
    mocks.attach_mock(self.mock_oslogin, 'oslogin')
    self.mock_utils.GetConfiguredUsers.return_value = configured
    self.mock_setup._GetAccountsData.return_value = desired
    self.mock_setup._GetEnableOsLoginValue.return_value = True
    self.mock_setup._GetEnableTwoFactorValue.return_value = False
    self.mock_oslogin.UpdateOsLogin.return_value = 0
    result = 'result'
    expected_add = []
    expected_remove = ['a', 'b', 'c']

    accounts_daemon.AccountsDaemon.HandleAccounts(self.mock_setup, result)
    expected_calls = [
        mock.call.setup.logger.debug(mock.ANY),
        mock.call.utils.GetConfiguredUsers(),
        mock.call.setup._GetEnableOsLoginValue(result),
        mock.call.setup._GetEnableTwoFactorValue(result),
        mock.call.oslogin.UpdateOsLogin(True, two_factor_desired=False),
        mock.call.setup._UpdateUsers(desired),
        mock.call.setup._RemoveUsers(mock.ANY),
        mock.call.utils.SetConfiguredUsers(mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)
    call_args, _ = self.mock_utils.SetConfiguredUsers.call_args
    self.assertEqual(set(call_args[0]), set(expected_add))
    call_args, _ = self.mock_setup._RemoveUsers.call_args
    self.assertEqual(set(call_args[0]), set(expected_remove))


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for config_manager.py module."""

from google_compute_engine import config_manager
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


def _HasOption(_, option):
  """Validate the option exists in the config file.

  Args:
    option: string, the config option to check.

  Returns:
    bool, True if test is not in the option name.
  """
  return 'test' not in option


def _HasSection(section):
  """Validate the section exists in the config file.

  Args:
    section: string, the config section to check.

  Returns:
    bool, True if test is not in the section name.
  """
  return 'test' not in section


class ConfigManagerTest(unittest.TestCase):

  option = 'option'
  section = 'section'
  value = 'value'

  def setUp(self):
    self.mock_config = mock.Mock()
    self.mock_config.has_option.side_effect = _HasOption
    self.mock_config.has_section.side_effect = _HasSection
    config_manager.parser.Parser = mock.Mock()
    config_manager.parser.Parser.return_value = self.mock_config

    self.config_file = 'test.cfg'
    self.config_header = 'Config file header.'

    self.mock_config_manager = config_manager.ConfigManager(
        config_file=self.config_file, config_header=self.config_header)

  def testAddHeader(self):
    mock_fp = mock.Mock()
    self.mock_config_manager._AddHeader(mock_fp)
    expected_calls = [
        mock.call('# %s' % self.config_header),
        mock.call('\n\n'),
    ]
    self.assertEqual(mock_fp.write.mock_calls, expected_calls)

  def testGetOptionString(self):
    self.mock_config_manager.GetOptionString(self.section, self.option)
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_option(self.section, self.option),
        mock.call.get(self.section, self.option),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testGetOptionStringNoOption(self):
    option = 'test-option'
    self.assertIsNone(
        self.mock_config_manager.GetOptionString(self.section, option))
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_option(self.section, option),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testGetOptionBool(self):
    self.mock_config_manager.GetOptionBool(self.section, self.option)
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_option(self.section, self.option),
        mock.call.getboolean(self.section, self.option),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testGetOptionBoolNoOption(self):
    option = 'test-option'
    self.assertTrue(
        self.mock_config_manager.GetOptionBool(self.section, option))
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_option(self.section, option),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testSetOption(self):
    self.mock_config_manager.SetOption(self.section, self.option, self.value)
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_section(self.section),
        mock.call.set(self.section, self.option, self.value),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testSetOptionNoOverwrite(self):
    self.mock_config_manager.SetOption(
        self.section, self.option, self.value, overwrite=False)
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_option(self.section, self.option),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testSetOptionNewSection(self):
    section = 'test-section'
    self.mock_config_manager.SetOption(section, self.option, self.value)
    expected_calls = [
        mock.call.read(self.config_file),
        mock.call.has_section(section),
        mock.call.add_section(section),
        mock.call.set(section, self.option, self.value),
    ]
    self.assertEqual(self.mock_config.mock_calls, expected_calls)

  def testWriteConfig(self):
    mock_open = mock.mock_open()
    with mock.patch('%s.open' % builtin, mock_open, create=False):
      self.mock_config_manager.WriteConfig()
      expected_calls = [
          mock.call('# %s' % self.config_header),
          mock.call('\n\n'),
      ]
      self.assertEqual(mock_open().write.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.config_manager.file_utils')
  def testWriteConfigNoHeader(self, mock_lock):
    self.mock_config_manager = config_manager.ConfigManager(
        config_file='/tmp/file.cfg')
    mock_open = mock.mock_open()
    with mock.patch('%s.open' % builtin, mock_open, create=False):
      self.mock_config_manager.WriteConfig()
      mock_open().write.assert_not_called()
    mock_lock.LockFile.assert_called_once_with('/var/lock/google_file.lock')

  @mock.patch('google_compute_engine.config_manager.file_utils')
  def testWriteConfigLocked(self, mock_lock):
    ioerror = IOError('Test Error')
    mock_lock.LockFile.side_effect = ioerror
    mock_open = mock.mock_open()
    with mock.patch('%s.open' % builtin, mock_open, create=False):
      with self.assertRaises(IOError) as error:
        self.mock_config_manager.WriteConfig()
      self.assertEqual(error.exception, ioerror)
      mock_open().write.assert_not_called()
    mock_lock.LockFile.assert_called_once_with('/var/lock/google_test.lock')


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities that are distro specific for use on FreeBSD 11."""

from google_compute_engine.distro_lib import helpers
from google_compute_engine.distro_lib import ip_forwarding_utils
from google_compute_engine.distro_lib import utils


class Utils(utils.Utils):
  """Utilities used by Linux guest services on FreeBSD 11."""

  def EnableIpv6(self, interfaces, logger, dhclient_script=None):
    """Configure the network interfaces for IPv6 using dhclient.

    Args:
      interface: string, the output device names for enabling IPv6.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    pass

  def EnableNetworkInterfaces(self, interfaces, logger, dhclient_script=None):
    """Enable the list of network interfaces.

    Args:
      interfaces: list of string, the output device names to enable.
      logger: logger object, used to write to SysLog and serial port.
      dhclient_script: string, the path to a dhclient script used by dhclient.
    """
    helpers.CallDhclient(interfaces, logger)

  def HandleClockSync(self, logger):
    """Sync the software clock with the hypervisor clock.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """
    helpers.CallNtpdate(logger)

  def IpForwardingUtils(self, logger, proto_id=None):
    """Get system IP address configuration utilities.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      proto_id: string, the routing protocol identifier for Google IP changes.
    """
    return ip_forwarding_utils.IpForwardingUtilsIfconfig(logger)



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for configuring IP address forwarding."""

import re
import subprocess
try:
  # The following modules are required by IpForwardingUtilsIfconfig.
  import netifaces
  import netaddr
except ImportError:
  netifaces = None
  netaddr = None


IP_REGEX = re.compile(r'\A(\d{1,3}\.){3}\d{1,3}\Z')
IP_ALIAS_REGEX = re.compile(r'\A(\d{1,3}\.){3}\d{1,3}/\d{1,2}\Z')


class IpForwardingUtilsBase(object):
  """System IP address configuration utilities."""

  def ParseForwardedIps(self, forwarded_ips):
    """Parse and validate forwarded IP addresses.

    Args:
      forwarded_ips: list, the IP address strings to parse.

    Returns:
      list, the valid IP address strings.
    """
    pass

  def GetForwardedIps(self, interface, interface_ip=None):
    """Retrieve the list of configured forwarded IP addresses.

    Args:
      interface: string, the output device to query.
      interface_ip: string, current interface ip address.

    Returns:
      list, the IP address strings.
    """
    pass

  def AddForwardedIp(self, address, interface):
    """Configure a new IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    pass

  def RemoveForwardedIp(self, address, interface):
    """Delete an IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    pass


class IpForwardingUtilsIproute(IpForwardingUtilsBase):
  """System IP address configuration utilities.

  Command used to add IPs:
    ip route add to local $IP/32 dev eth0 proto 66
  Command used to fetch list of configured IPs:
    ip route ls table local type local dev eth0 scope host proto 66
  """

  def __init__(self, logger, proto_id=None):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      proto_id: string, the routing protocol identifier for Google IP changes.
    """
    self.logger = logger
    self.proto_id = proto_id or '66'

  def _CreateRouteOptions(self, **kwargs):
    """Create a dictionary of parameters to append to the ip route command.

    Args:
      **kwargs: dict, the string parameters to update in the ip route command.

    Returns:
      dict, the string parameters to append to the ip route command.
    """
    options = {
        'proto': self.proto_id,
        'scope': 'host',
    }
    options.update(kwargs)
    return options

  def _RunIpRoute(self, args=None, options=None):
    """Run a command with ip route and return the response.

    Args:
      args: list, the string ip route command args to execute.
      options: dict, the string parameters to append to the ip route command.

    Returns:
      string, the standard output from the ip route command execution.
    """
    args = args or []
    options = options or {}
    command = ['ip', 'route']
    command.extend(args)
    for item in options.items():
      command.extend(item)
    try:
      process = subprocess.Popen(
          command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
      stdout, stderr = process.communicate()
    except OSError as e:
      self.logger.warning('Exception running %s. %s.', command, str(e))
    else:
      if process.returncode:
        message = 'Non-zero exit status running %s. %s.'
        self.logger.warning(message, command, stderr.strip())
      else:
        return stdout.decode('utf-8', 'replace')
    return ''

  def ParseForwardedIps(self, forwarded_ips):
    """Parse and validate forwarded IP addresses.

    Args:
      forwarded_ips: list, the IP address strings to parse.

    Returns:
      list, the valid IP address strings.
    """
    addresses = []
    forwarded_ips = forwarded_ips or []
    for ip in forwarded_ips:
      if ip and (IP_REGEX.match(ip) or IP_ALIAS_REGEX.match(ip)):
        addresses.append(ip[:-3] if ip.endswith('/32') else ip)
      else:
        self.logger.warning('Could not parse IP address: "%s".', ip)
    return addresses

  def GetForwardedIps(self, interface, interface_ip=None):
    """Retrieve the list of configured forwarded IP addresses.

    Args:
      interface: string, the output device to query.
      interface_ip: string, current interface ip address.

    Returns:
      list, the IP address strings.
    """
    args = ['ls', 'table', 'local', 'type', 'local']
    options = self._CreateRouteOptions(dev=interface)
    result = self._RunIpRoute(args=args, options=options)
    result = re.sub(r'local\s', r'', result)
    return self.ParseForwardedIps(result.split())

  def AddForwardedIp(self, address, interface):
    """Configure a new IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    address = address if IP_ALIAS_REGEX.match(address) else '%s/32' % address
    args = ['add', 'to', 'local', address]
    options = self._CreateRouteOptions(dev=interface)
    self._RunIpRoute(args=args, options=options)

  def RemoveForwardedIp(self, address, interface):
    """Delete an IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    address = address if IP_ALIAS_REGEX.match(address) else '%s/32' % address
    args = ['delete', 'to', 'local', address]
    options = self._CreateRouteOptions(dev=interface)
    self._RunIpRoute(args=args, options=options)


class IpForwardingUtilsIfconfig(IpForwardingUtilsBase):
  """System IP address configuration utilities."""

  def __init__(self, logger):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
    """

    self.logger = logger

  def _RunIfconfig(self, args=None, options=None):
    """Run a command with ifconfig and return the response.

    Args:
      args: list, the string ip route command args to execute.
      options: dict, the string parameters to append to the ip route command.

    Returns:
      string, the standard output from the ip route command execution.
    """
    args = args or []
    options = options or {}
    command = ['ifconfig']
    command.extend(args)
    for item in options.items():
      command.extend(item)
    try:
      process = subprocess.Popen(
          command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
      stdout, stderr = process.communicate()
    except OSError as e:
      self.logger.warning('Exception running %s. %s.', command, str(e))
    else:
      if process.returncode:
        message = 'Non-zero exit status running %s. %s.'
        self.logger.warning(message, command, stderr.strip())
      else:
        return stdout.decode('utf-8', 'replace')
    return ''

  def ParseForwardedIps(self, forwarded_ips):
    """Parse and validate forwarded IP addresses.

    Args:
      forwarded_ips: list, the IP address strings to parse.

    Returns:
      list, the valid IP address strings.
    """
    addresses = []
    forwarded_ips = forwarded_ips or []
    for ip in forwarded_ips:
      if ip and (IP_REGEX.match(ip) or IP_ALIAS_REGEX.match(ip)):
        addresses.extend([str(addr) for addr in list(netaddr.IPNetwork(ip))])
      else:
        self.logger.warning('Could not parse IP address: "%s".', ip)
    return addresses

  def GetForwardedIps(self, interface, interface_ip=None):
    """Retrieve the list of configured forwarded IP addresses.

    Args:
      interface: string, the output device to query.
      interface_ip: string, current interface ip address.

    Returns:
      list, the IP address strings.
    """
    try:
      ips = netifaces.ifaddresses(interface)
      ips = ips[netifaces.AF_INET]
    except (ValueError, IndexError):
      return []
    forwarded_ips = []
    for ip in ips:
      if ip['addr'] != interface_ip:
        full_addr = '%s/%d' % (ip['addr'], netaddr.IPAddress(ip['netmask']).netmask_bits())
        forwarded_ips.append(full_addr)
    return self.ParseForwardedIps(forwarded_ips)

  def AddForwardedIp(self, address, interface):
    """Configure a new IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    for ip in list(netaddr.IPNetwork(address)):
      self._RunIfconfig(args=[interface, 'alias', '%s/32' % str(ip)])

  def RemoveForwardedIp(self, address, interface):
    """Delete an IP address on the network interface.

    Args:
      address: string, the IP address to configure.
      interface: string, the output device to use.
    """
    ip = netaddr.IPNetwork(address)
    self._RunIfconfig(args=[interface, '-alias', str(ip.ip)])



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for utils.py module."""

import os
import shutil
import tempfile

from google_compute_engine.distro_lib.el_7 import utils
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class UtilsTest(unittest.TestCase):

  def setUp(self):
    # Create a temporary directory.
    self.test_dir = tempfile.mkdtemp()
    self.mock_logger = mock.Mock()
    self.mock_setup = mock.create_autospec(utils.Utils)
    self.mock_setup.network_path = '/etc/sysconfig/network-scripts'

  def tearDown(self):
    # Remove the directory after the test.
    shutil.rmtree(self.test_dir)

  def testModifyInterface(self):
    config_file = os.path.join(self.test_dir, 'config.cfg')
    config_content = [
        '# File comment.\n',
        'A="apple"\n',
        'B=banana\n',
        'B=banana\n',
    ]
    with open(config_file, 'w') as config:
      for line in config_content:
        config.write(line)

    # Write a value for an existing config without overriding it.
    utils.Utils._ModifyInterface(
        self.mock_setup, config_file, 'A', 'aardvark', replace=False)
    self.assertEqual(open(config_file).readlines(), config_content)
    # Write a value for a config that is not already set.
    utils.Utils._ModifyInterface(
        self.mock_setup, config_file, 'C', 'none', replace=False)
    config_content.append('C=none\n')
    self.assertEqual(open(config_file).readlines(), config_content)
    # Write a value for an existing config with replacement.
    utils.Utils._ModifyInterface(
        self.mock_setup, config_file, 'A', 'aardvark', replace=True)
    config_content[1] = 'A=aardvark\n'
    self.assertEqual(open(config_file).readlines(), config_content)
    # Write a value for an existing config with multiple occurrences.
    utils.Utils._ModifyInterface(
        self.mock_setup, config_file, 'B', '"banana"', replace=True)
    config_content[2] = config_content[3] = 'B="banana"\n'
    self.assertEqual(open(config_file).readlines(), config_content)

  @mock.patch('google_compute_engine.distro_lib.el_7.utils.os.path.exists')
  def testDisableNetworkManager(self, mock_exists):
    mock_open = mock.mock_open()
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_open, 'open')
    mocks.attach_mock(self.mock_logger, 'logger')
    mocks.attach_mock(self.mock_setup._ModifyInterface, 'modify')
    mock_exists.side_effect = [True, False]

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      utils.Utils._DisableNetworkManager(
          self.mock_setup, ['eth0', 'eth1'], self.mock_logger)
      expected_calls = [
          mock.call.exists('/etc/sysconfig/network-scripts/ifcfg-eth0'),
          mock.call.modify(mock.ANY, 'DEVICE', 'eth0', replace=False),
          mock.call.modify(mock.ANY, 'NM_CONTROLLED', 'no', replace=True),
          mock.call.exists('/etc/sysconfig/network-scripts/ifcfg-eth1'),
          mock.call.open('/etc/sysconfig/network-scripts/ifcfg-eth1', 'w'),
          mock.call.open().__enter__(),
          mock.call.open().write(mock.ANY),
          mock.call.open().__exit__(None, None, None),
          mock.call.logger.info(mock.ANY, 'eth1'),
      ]
      self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallDhclientIpv6')
  @mock.patch('google_compute_engine.distro_lib.helpers.CallEnableRouteAdvertisements')
  def testEnableIpv6(self, mock_call_enable_ra, mock_call_dhclient):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call_dhclient, 'dhclient')
    mocks.attach_mock(mock_call_enable_ra, 'enable_ra')

    utils.Utils.EnableIpv6(self.mock_setup, ['A', 'B'], self.mock_logger)
    expected_calls = [
        mock.call.enable_ra(['A', 'B'], mock.ANY),
        mock.call.dhclient(['A', 'B'], mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallDhclientIpv6')
  def testDisableIpv6(self, mock_call_dhclient):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call_dhclient, 'dhclient')

    utils.Utils.DisableIpv6(self.mock_setup, ['A', 'B'], self.mock_logger)
    expected_calls = [
        mock.call.dhclient(['A', 'B'], mock.ANY, None, release_lease=True),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.el_7.utils.os.path.exists')
  @mock.patch('google_compute_engine.distro_lib.helpers.CallDhclient')
  def testEnableNetworkInterfaces(self, mock_call, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_logger, 'logger')
    mocks.attach_mock(self.mock_setup._DisableNetworkManager, 'disable')
    mocks.attach_mock(mock_call, 'call_dhclient')
    mock_exists.side_effect = [True, False]

    # Enable interfaces with network manager enabled.
    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['A', 'B'], self.mock_logger)
    # Enable interfaces with network manager is not present.
    utils.Utils.EnableNetworkInterfaces(
        self.mock_setup, ['C', 'D'], self.mock_logger)
    expected_calls = [
        mock.call.exists('/etc/sysconfig/network-scripts'),
        mock.call.disable(['A', 'B'], mock.ANY),
        mock.call.call_dhclient(['A', 'B'], mock.ANY),
        mock.call.exists('/etc/sysconfig/network-scripts'),
        mock.call.call_dhclient(['C', 'D'], mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.CallHwclock')
  def testHandleClockSync(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.HandleClockSync(self.mock_setup, self.mock_logger)
    expected_calls = [mock.call.call(mock.ANY)]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.ip_forwarding_utils.IpForwardingUtilsIproute')
  def testIpForwardingUtils(self, mock_call):
    mocks = mock.Mock()
    mocks.attach_mock(mock_call, 'call')

    utils.Utils.IpForwardingUtils(self.mock_setup, self.mock_logger, '66')
    expected_calls = [mock.call.call(mock.ANY, '66')]
    self.assertEqual(mocks.mock_calls, expected_calls)



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for provisioning or deprovisioning a Linux user account."""

import grp
import os
import pwd
import re
import shutil
import subprocess
import tempfile

from google_compute_engine import constants
from google_compute_engine import file_utils

USER_REGEX = re.compile(r'\A[A-Za-z0-9._][A-Za-z0-9._-]{0,31}\Z')
DEFAULT_GPASSWD_ADD_CMD = 'gpasswd -a {user} {group}'
DEFAULT_GPASSWD_REMOVE_CMD = 'gpasswd -d {user} {group}'
DEFAULT_GROUPADD_CMD = 'groupadd {group}'
DEFAULT_USERADD_CMD = 'useradd -m -s /bin/bash -p * {user}'
DEFAULT_USERDEL_CMD = 'userdel -r {user}'
DEFAULT_USERMOD_CMD = 'usermod -G {groups} {user}'


class AccountsUtils(object):
  """System user account configuration utilities."""

  google_comment = '# Added by Google'

  def __init__(
      self, logger, groups=None, remove=False, gpasswd_add_cmd=None,
      gpasswd_remove_cmd=None, groupadd_cmd=None, useradd_cmd=None,
      userdel_cmd=None, usermod_cmd=None):
    """Constructor.

    Args:
      logger: logger object, used to write to SysLog and serial port.
      groups: string, a comma separated list of groups.
      remove: bool, True if deprovisioning a user should be destructive.
      gpasswd_add_cmd: string, command to add an user to a group.
      gpasswd_remove_cmd: string, command to remove an user from a group.
      groupadd_cmd: string, command to add a new group.
      useradd_cmd: string, command to create a new user.
      userdel_cmd: string, command to delete a user.
      usermod_cmd: string, command to modify user's groups.
    """
    self.gpasswd_add_cmd = gpasswd_add_cmd or DEFAULT_GPASSWD_ADD_CMD
    self.gpasswd_remove_cmd = gpasswd_remove_cmd or DEFAULT_GPASSWD_REMOVE_CMD
    self.groupadd_cmd = groupadd_cmd or DEFAULT_GROUPADD_CMD
    self.useradd_cmd = useradd_cmd or DEFAULT_USERADD_CMD
    self.userdel_cmd = userdel_cmd or DEFAULT_USERDEL_CMD
    self.usermod_cmd = usermod_cmd or DEFAULT_USERMOD_CMD
    self.logger = logger
    self.google_sudoers_group = 'google-sudoers'
    self.google_sudoers_file = (
        constants.LOCALBASE + '/etc/sudoers.d/google_sudoers')
    self.google_users_dir = constants.LOCALBASE + '/var/lib/google'
    self.google_users_file = os.path.join(self.google_users_dir, 'google_users')

    self._CreateSudoersGroup()
    self.groups = groups.split(',') if groups else []
    self.groups = list(filter(self._GetGroup, self.groups))
    self.remove = remove

  def _GetGroup(self, group):
    """Retrieve a Linux group.

    Args:
      group: string, the name of the Linux group to retrieve.

    Returns:
      grp.struct_group, the Linux group or None if it does not exist.
    """
    try:
      return grp.getgrnam(group)
    except KeyError:
      return None

  def _CreateSudoersGroup(self):
    """Create a Linux group for Google added sudo user accounts."""
    if not self._GetGroup(self.google_sudoers_group):
      try:
        command = self.groupadd_cmd.format(group=self.google_sudoers_group)
        subprocess.check_call(command.split(' '))
      except subprocess.CalledProcessError as e:
        self.logger.warning('Could not create the sudoers group. %s.', str(e))

    if not os.path.exists(self.google_sudoers_file):
      try:
        with open(self.google_sudoers_file, 'w') as group:
          message = '%{0} ALL=(ALL:ALL) NOPASSWD:ALL'.format(
              self.google_sudoers_group)
          group.write(message)
      except IOError as e:
        self.logger.error(
            'Could not write sudoers file. %s. %s',
            self.google_sudoers_file, str(e))
        return

    file_utils.SetPermissions(
        self.google_sudoers_file, mode=0o440, uid=0, gid=0)

  def _GetUser(self, user):
    """Retrieve a Linux user account.

    Args:
      user: string, the name of the Linux user account to retrieve.

    Returns:
      pwd.struct_passwd, the Linux user or None if it does not exist.
    """
    try:
      return pwd.getpwnam(user)
    except KeyError:
      return None

  def _AddUser(self, user):
    """Configure a Linux user account.

    Args:
      user: string, the name of the Linux user account to create.

    Returns:
      bool, True if user creation succeeded.
    """
    self.logger.info('Creating a new user account for %s.', user)

    command = self.useradd_cmd.format(user=user)
    try:
      subprocess.check_call(command.split(' '))
    except subprocess.CalledProcessError as e:
      self.logger.warning('Could not create user %s. %s.', user, str(e))
      return False
    else:
      self.logger.info('Created user account %s.', user)
      return True

  def _UpdateUserGroups(self, user, groups):
    """Update group membership for a Linux user.

    Args:
      user: string, the name of the Linux user account.
      groups: list, the group names to add the user as a member.

    Returns:
      bool, True if user update succeeded.
    """
    groups = ','.join(groups)
    self.logger.debug('Updating user %s with groups %s.', user, groups)
    command = self.usermod_cmd.format(user=user, groups=groups)
    try:
      subprocess.check_call(command.split(' '))
    except subprocess.CalledProcessError as e:
      self.logger.warning('Could not update user %s. %s.', user, str(e))
      return False
    else:
      self.logger.debug('Updated user account %s.', user)
      return True

  def _UpdateAuthorizedKeys(self, user, ssh_keys):
    """Update the authorized keys file for a Linux user with a list of SSH keys.

    Args:
      user: string, the name of the Linux user account.
      ssh_keys: list, the SSH key strings associated with the user.

    Raises:
      IOError, raised when there is an exception updating a file.
      OSError, raised when setting permissions or writing to a read-only
          file system.
    """
    pw_entry = self._GetUser(user)
    if not pw_entry:
      return

    uid = pw_entry.pw_uid
    gid = pw_entry.pw_gid
    home_dir = pw_entry.pw_dir
    ssh_dir = os.path.join(home_dir, '.ssh')

    # Not all sshd's support multiple authorized_keys files so we have to
    # share one with the user. We add each of our entries as follows:
    #  # Added by Google
    #  authorized_key_entry
    authorized_keys_file = os.path.join(ssh_dir, 'authorized_keys')

    # Do not write to the authorized keys file if it is a symlink.
    if os.path.islink(ssh_dir) or os.path.islink(authorized_keys_file):
      self.logger.warning(
          'Not updating authorized keys for user %s. File is a symlink.', user)
      return

    # Create home directory if it does not exist. This can happen if _GetUser
    # (getpwnam) returns non-local user info (e.g., from LDAP).
    if not os.path.exists(home_dir):
        file_utils.SetPermissions(home_dir, mode=0o755, uid=uid, gid=gid,
            mkdir=True)

    # Create ssh directory if it does not exist.
    file_utils.SetPermissions(ssh_dir, mode=0o700, uid=uid, gid=gid, mkdir=True)

    # Create entry in the authorized keys file.
    prefix = self.logger.name + '-'
    with tempfile.NamedTemporaryFile(
        mode='w', prefix=prefix, delete=True) as updated_keys:
      updated_keys_file = updated_keys.name
      if os.path.exists(authorized_keys_file):
        lines = open(authorized_keys_file).readlines()
      else:
        lines = []

      google_lines = set()
      for i, line in enumerate(lines):
        if line.startswith(self.google_comment):
          google_lines.update([i, i+1])

      # Write user's authorized key entries.
      for i, line in enumerate(lines):
        if i not in google_lines and line:
          line += '\n' if not line.endswith('\n') else ''
          updated_keys.write(line)

      # Write the Google authorized key entries at the end of the file.
      # Each entry is preceded by '# Added by Google'.
      for ssh_key in ssh_keys:
        ssh_key += '\n' if not ssh_key.endswith('\n') else ''
        updated_keys.write('%s\n' % self.google_comment)
        updated_keys.write(ssh_key)

      # Write buffered data to the updated keys file without closing it and
      # update the Linux user's authorized keys file.
      updated_keys.flush()
      shutil.copy(updated_keys_file, authorized_keys_file)

    file_utils.SetPermissions(
        authorized_keys_file, mode=0o600, uid=uid, gid=gid)

  def _UpdateSudoer(self, user, sudoer=False):
    """Update sudoer group membership for a Linux user account.

    Args:
      user: string, the name of the Linux user account.
      sudoer: bool, True if the user should be a sudoer.

    Returns:
      bool, True if user update succeeded.
    """
    if sudoer:
      self.logger.info('Adding user %s to the Google sudoers group.', user)
      command = self.gpasswd_add_cmd.format(
          user=user, group=self.google_sudoers_group)
    else:
      self.logger.info('Removing user %s from the Google sudoers group.', user)
      command = self.gpasswd_remove_cmd.format(
          user=user, group=self.google_sudoers_group)

    try:
      subprocess.check_call(command.split(' '))
    except subprocess.CalledProcessError as e:
      self.logger.warning('Could not update user %s. %s.', user, str(e))
      return False
    else:
      self.logger.debug('Removed user %s from the Google sudoers group.', user)
      return True

  def _RemoveAuthorizedKeys(self, user):
    """Remove a Linux user account's authorized keys file to prevent login.

    Args:
      user: string, the Linux user account to remove access.
    """
    pw_entry = self._GetUser(user)
    if not pw_entry:
      return

    home_dir = pw_entry.pw_dir
    authorized_keys_file = os.path.join(home_dir, '.ssh', 'authorized_keys')
    if os.path.exists(authorized_keys_file):
      try:
        os.remove(authorized_keys_file)
      except OSError as e:
        message = 'Could not remove authorized keys for user %s. %s.'
        self.logger.warning(message, user, str(e))

  def GetConfiguredUsers(self):
    """Retrieve the list of configured Google user accounts.

    Returns:
      list, the username strings of users congfigured by Google.
    """
    if os.path.exists(self.google_users_file):
      users = open(self.google_users_file).readlines()
    else:
      users = []
    return [user.strip() for user in users]

  def SetConfiguredUsers(self, users):
    """Set the list of configured Google user accounts.

    Args:
      users: list, the username strings of the Linux accounts.
    """
    prefix = self.logger.name + '-'
    with tempfile.NamedTemporaryFile(
        mode='w', prefix=prefix, delete=True) as updated_users:
      updated_users_file = updated_users.name
      for user in users:
        updated_users.write(user + '\n')
      updated_users.flush()
      if not os.path.exists(self.google_users_dir):
        os.makedirs(self.google_users_dir)
      shutil.copy(updated_users_file, self.google_users_file)

    file_utils.SetPermissions(self.google_users_file, mode=0o600, uid=0, gid=0)

  def UpdateUser(self, user, ssh_keys):
    """Update a Linux user with authorized SSH keys.

    Args:
      user: string, the name of the Linux user account.
      ssh_keys: list, the SSH key strings associated with the user.

    Returns:
      bool, True if the user account updated successfully.
    """
    if not bool(USER_REGEX.match(user)):
      self.logger.warning('Invalid user account name %s.', user)
      return False
    if not self._GetUser(user):
      # User does not exist. Attempt to create the user and add them to the
      # appropriate user groups.
      if not (self._AddUser(user)
              and self._UpdateUserGroups(user, self.groups)):
        return False
    # Add the user to the google sudoers group.
    if not self._UpdateSudoer(user, sudoer=True):
      return False

    # Don't try to manage account SSH keys with a shell set to disable
    # logins. This helps avoid problems caused by operator and root sharing
    # a home directory in CentOS and RHEL.
    pw_entry = self._GetUser(user)
    if pw_entry and os.path.basename(pw_entry.pw_shell) == 'nologin':
      message = 'Not updating user %s. User set `nologin` as login shell.'
      self.logger.debug(message, user)
      return True

    try:
      self._UpdateAuthorizedKeys(user, ssh_keys)
    except (IOError, OSError) as e:
      message = 'Could not update the authorized keys file for user %s. %s.'
      self.logger.warning(message, user, str(e))
      return False
    else:
      return True

  def RemoveUser(self, user):
    """Remove a Linux user account.

    Args:
      user: string, the Linux user account to remove.
    """
    self.logger.info('Removing user %s.', user)
    if self.remove:
      command = self.userdel_cmd.format(user=user)
      try:
        subprocess.check_call(command.split(' '))
      except subprocess.CalledProcessError as e:
        self.logger.warning('Could not remove user %s. %s.', user, str(e))
      else:
        self.logger.info('Removed user account %s.', user)
    self._RemoveAuthorizedKeys(user)
    self._UpdateSudoer(user, sudoer=False)



#!/usr/bin/python
# Copyright 2018 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for helpers.py module."""

import subprocess

from google_compute_engine.distro_lib import helpers
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class HelpersTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()

  @mock.patch('google_compute_engine.distro_lib.helpers.os.path.exists')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallDhclient(self, mock_call, mock_exists):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(self.mock_logger, 'logger')

    mock_exists.side_effect = [False, True]
    mock_call.side_effect = [
        None, None, None, None, None, None,
        subprocess.CalledProcessError(1, 'Test'),
    ]

    helpers.CallDhclient(['a', 'b'], self.mock_logger, 'test_script')
    helpers.CallDhclient(['c', 'd'], self.mock_logger, 'test_script')
    helpers.CallDhclient(['e', 'f'], self.mock_logger, None)
    helpers.CallDhclient(['g', 'h'], self.mock_logger, None)

    expected_calls = [
        mock.call.logger.info(mock.ANY, ['a', 'b']),
        mock.call.exists('test_script'),
        mock.call.call(['dhclient', '-x', 'a', 'b']),
        mock.call.call(['dhclient', 'a', 'b']),
        mock.call.logger.info(mock.ANY, ['c', 'd']),
        mock.call.exists('test_script'),
        mock.call.call(['dhclient', '-sf', 'test_script', '-x', 'c', 'd']),
        mock.call.call(['dhclient', '-sf', 'test_script', 'c', 'd']),
        mock.call.logger.info(mock.ANY, ['e', 'f']),
        mock.call.call(['dhclient', '-x', 'e', 'f']),
        mock.call.call(['dhclient', 'e', 'f']),
        mock.call.logger.info(mock.ANY, ['g', 'h']),
        mock.call.call(['dhclient', '-x', 'g', 'h']),
        mock.call.logger.warning(mock.ANY, ['g', 'h']),
    ]

    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.os.path.exists')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallDhclientIpv6NonExistentScript(self, mock_call, mock_exists):
    mock_logger = mock.Mock()

    mock_exists.side_effect = [False]
    helpers.CallDhclientIpv6(['a', 'b'], mock_logger, 'test_script')
    mock_call.assert_has_calls(
        [
            mock.call.call(
                ['timeout', '5', 'dhclient', '-1', '-6', '-v', 'a', 'b']),
        ])

  @mock.patch('google_compute_engine.distro_lib.helpers.os.path.exists')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallDhclientIpv6(self, mock_call, mock_exists):
    mock_logger = mock.Mock()
    mock_exists.side_effect = [True]
    mock_call.side_effect = [
        None,
        None,
        subprocess.CalledProcessError(1, 'Test'),
        None,
        None,
        subprocess.CalledProcessError(1, 'Test'),
    ]

    helpers.CallDhclientIpv6(['a', 'b'], mock_logger, 'test_script')
    helpers.CallDhclientIpv6(['c', 'd'], mock_logger, None)
    helpers.CallDhclientIpv6(['e', 'f'], mock_logger, None)
    helpers.CallDhclientIpv6(
        ['g', 'h'], mock_logger, 'test_script', release_lease=True)
    helpers.CallDhclientIpv6(['i', 'j'], mock_logger, None, release_lease=True)
    helpers.CallDhclientIpv6(['k', 'l'], mock_logger, None, release_lease=True)

    expected_calls = [
        mock.call.call(
            [
                'timeout', '5','dhclient', '-sf', 'test_script', '-1', '-6',
                '-v', 'a', 'b',
            ]),
        mock.call.call(
            [
                'timeout', '5', 'dhclient', '-1', '-6', '-v', 'c', 'd',
            ]),
        mock.call.call(
            [
                'timeout', '5', 'dhclient', '-1', '-6', '-v', 'e', 'f',
             ]),
        mock.call.call(
            [
                'timeout', '5', 'dhclient', '-6', '-r', '-v', 'g', 'h',
            ]),
        mock.call.call(
            [
                'timeout', '5', 'dhclient', '-6', '-r', '-v', 'i', 'j',
            ]),
        mock.call.call(
            [
                'timeout', '5', 'dhclient', '-6', '-r', '-v', 'k', 'l',
            ]),
    ]

    self.assertEqual(mock_call.mock_calls, expected_calls)
    mock_logger.assert_has_calls(
        [
            mock.call.warning(mock.ANY, ['e', 'f']),
        ])
    mock_logger.assert_has_calls(
        [
            mock.call.warning(mock.ANY, ['k', 'l']),
        ])

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testEnableRouteAdvertisements(self, mock_call):
    mock_logger = mock.Mock()
    interfaces = ['foo', 'bar', 'baz']
    helpers.CallEnableRouteAdvertisements(interfaces, mock_logger)
    mock_call.assert_has_calls([
        mock.call(
            [
                'sysctl', '-w',
                'net.ipv6.conf.%s.accept_ra_rt_info_max_plen=128' % interface,
            ])
        for interface in interfaces])

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallHwclock(self, mock_call):
    command = ['/sbin/hwclock', '--hctosys']
    mock_logger = mock.Mock()

    helpers.CallHwclock(mock_logger)
    mock_call.assert_called_once_with(command)
    expected_calls = [mock.call.info(mock.ANY)]
    self.assertEqual(mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallHwclockError(self, mock_call):
    command = ['/sbin/hwclock', '--hctosys']
    mock_logger = mock.Mock()
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    helpers.CallHwclock(mock_logger)
    mock_call.assert_called_once_with(command)
    expected_calls = [mock.call.warning(mock.ANY)]
    self.assertEqual(mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.call')
  def testCallNtpdateActive(self, mock_call, mock_check_call):
    command_status = ['service', 'ntpd', 'status']
    command_stop = ['service', 'ntpd', 'stop']
    command_start = ['service', 'ntpd', 'start']
    command_ntpdate = 'ntpdate `awk \'$1=="server" {print $2}\' /etc/ntp.conf`'
    mock_logger = mock.Mock()
    mock_call.return_value = 0
    mock_check_call.return_value = True

    helpers.CallNtpdate(mock_logger)
    mock_call.assert_called_once_with(command_status)
    expected_calls = [
        mock.call(command_stop),
        mock.call(command_ntpdate, shell=True),
        mock.call(command_start),
    ]
    self.assertEqual(mock_check_call.mock_calls, expected_calls)
    expected_calls = [mock.call.info(mock.ANY)]
    self.assertEqual(mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.call')
  def testCallNtpdateInactive(self, mock_call, mock_check_call):
    command_status = ['service', 'ntpd', 'status']
    command_ntpdate = 'ntpdate `awk \'$1=="server" {print $2}\' /etc/ntp.conf`'
    mock_logger = mock.Mock()
    mock_call.return_value = 1

    helpers.CallNtpdate(mock_logger)
    mock_call.assert_called_once_with(command_status)
    mock_check_call.assert_called_once_with(command_ntpdate, shell=True)
    expected_calls = [mock.call.info(mock.ANY)]
    self.assertEqual(mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.call')
  def testCallNtpdateError(self, mock_call, mock_check_call):
    command_status = ['service', 'ntpd', 'status']
    command_ntpdate = 'ntpdate `awk \'$1=="server" {print $2}\' /etc/ntp.conf`'
    mock_logger = mock.Mock()
    mock_check_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    helpers.CallNtpdate(mock_logger)
    mock_call.assert_called_once_with(command_status)
    mock_check_call.assert_called_once_with(command_ntpdate, shell=True)
    expected_calls = [mock.call.warning(mock.ANY)]
    self.assertEqual(mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.distro_lib.helpers.subprocess.check_call')
  def testCallSysctl(self, mock_call):
    command = ['sysctl', '-w']
    mock_logger = mock.Mock()
    expected_log_calls = []
    for name in ['foo', 'bar', 'baz']:
      for value in ['foo', 'bar', 'baz']:
        params = ['{name}={value}'.format(name=name, value=value)]
        helpers.CallSysctl(mock_logger, name, value)
        mock_call.assert_called_with(command + params)
        expected_log_calls.append(mock.call.info(mock.ANY, name))
    self.assertEqual(mock_logger.mock_calls, expected_log_calls)

    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')
    helpers.CallSysctl(mock_logger, 'fail', 1)
    mock_logger.assert_has_calls([mock.call.warning(mock.ANY, 'fail')])



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Manage the retrieval and excution of metadata scripts."""

import contextlib
import logging.handlers
import optparse
import shutil
import tempfile

from google_compute_engine import config_manager
from google_compute_engine import logger
from google_compute_engine.metadata_scripts import script_executor
from google_compute_engine.metadata_scripts import script_retriever


@contextlib.contextmanager
def _CreateTempDir(prefix, run_dir=None):
  """Context manager for creating a temporary directory.

  Args:
    prefix: string, the prefix for the temporary directory.
    run_dir: string, the base directory location of the temporary directory.

  Yields:
    string, the temporary directory created.
  """
  temp_dir = tempfile.mkdtemp(prefix=prefix + '-', dir=run_dir)
  try:
    yield temp_dir
  finally:
    shutil.rmtree(temp_dir)


class ScriptManager(object):
  """A class for retrieving and executing metadata scripts."""

  def __init__(
      self, script_type, default_shell=None, run_dir=None, debug=False):
    """Constructor.

    Args:
      script_type: string, the metadata script type to run.
      default_shell: string, the default shell to execute the script.
      run_dir: string, the base directory location of the temporary directory.
      debug: bool, True if debug output should write to the console.
    """
    self.script_type = script_type
    self.default_shell = default_shell
    name = '%s-script' % self.script_type
    facility = logging.handlers.SysLogHandler.LOG_DAEMON
    self.logger = logger.Logger(name=name, debug=debug, facility=facility)
    self.retriever = script_retriever.ScriptRetriever(self.logger, script_type)
    self.executor = script_executor.ScriptExecutor(
        self.logger, script_type, default_shell=default_shell)
    self._RunScripts(run_dir=run_dir)

  def _RunScripts(self, run_dir=None):
    """Retrieve metadata scripts and execute them.

    Args:
      run_dir: string, the base directory location of the temporary directory.
    """
    with _CreateTempDir(self.script_type, run_dir=run_dir) as dest_dir:
      try:
        self.logger.info('Starting %s scripts.', self.script_type)
        script_dict = self.retriever.GetScripts(dest_dir)
        self.executor.RunScripts(script_dict)
      finally:
        self.logger.info('Finished running %s scripts.', self.script_type)


def main():
  script_types = ('startup', 'shutdown')
  parser = optparse.OptionParser()
  parser.add_option(
      '-d', '--debug', action='store_true', dest='debug',
      help='print debug output to the console.')
  parser.add_option(
      '--script-type', dest='script_type', help='metadata script type.')
  (options, _) = parser.parse_args()
  if options.script_type and options.script_type.lower() in script_types:
    script_type = options.script_type.lower()
  else:
    valid_args = ', '.join(script_types)
    message = 'No valid argument specified. Options: [%s].' % valid_args
    raise ValueError(message)

  instance_config = config_manager.ConfigManager()
  if instance_config.GetOptionBool('MetadataScripts', script_type):
    ScriptManager(
        script_type,
        default_shell=instance_config.GetOptionString(
            'MetadataScripts', 'default_shell'),
        run_dir=instance_config.GetOptionString('MetadataScripts', 'run_dir'),
        debug=bool(options.debug))


if __name__ == '__main__':
  main()



#!/usr/bin/python
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Unittest for accounts_utils.py module."""

import subprocess

from google_compute_engine.accounts import accounts_utils
from google_compute_engine.test_compat import builtin
from google_compute_engine.test_compat import mock
from google_compute_engine.test_compat import unittest


class AccountsUtilsTest(unittest.TestCase):

  def setUp(self):
    self.mock_logger = mock.Mock()
    self.sudoers_group = 'google-sudoers'
    self.sudoers_file = '/sudoers/file'
    self.users_dir = '/users'
    self.users_file = '/users/file'
    self.gpasswd_add_cmd = 'gpasswd -a {user} {group}'
    self.gpasswd_remove_cmd = 'gpasswd -d {user} {group}'
    self.groupadd_cmd = 'groupadd {group}'
    self.useradd_cmd = 'useradd -m -s /bin/bash -p * {user}'
    self.userdel_cmd = 'userdel -r {user}'
    self.usermod_cmd = 'usermod -G {groups} {user}'

    self.mock_utils = mock.create_autospec(accounts_utils.AccountsUtils)
    self.mock_utils.google_comment = accounts_utils.AccountsUtils.google_comment
    self.mock_utils.google_sudoers_group = self.sudoers_group
    self.mock_utils.google_sudoers_file = self.sudoers_file
    self.mock_utils.google_users_dir = self.users_dir
    self.mock_utils.google_users_file = self.users_file
    self.mock_utils.logger = self.mock_logger
    self.mock_utils.gpasswd_add_cmd = self.gpasswd_add_cmd
    self.mock_utils.gpasswd_remove_cmd = self.gpasswd_remove_cmd
    self.mock_utils.groupadd_cmd = self.groupadd_cmd
    self.mock_utils.useradd_cmd = self.useradd_cmd
    self.mock_utils.userdel_cmd = self.userdel_cmd
    self.mock_utils.usermod_cmd = self.usermod_cmd

  @mock.patch('google_compute_engine.accounts.accounts_utils.AccountsUtils._GetGroup')
  @mock.patch('google_compute_engine.accounts.accounts_utils.AccountsUtils._CreateSudoersGroup')
  def testAccountsUtils(self, mock_create, mock_group):
    mock_logger = mock.Mock()
    mock_group.side_effect = lambda group: 'google' in group

    utils = accounts_utils.AccountsUtils(
        logger=mock_logger, groups='foo,google,bar', remove=True)
    mock_create.assert_called_once_with()
    self.assertEqual(utils.logger, mock_logger)
    self.assertEqual(sorted(utils.groups), ['google'])
    self.assertTrue(utils.remove)

  @mock.patch('google_compute_engine.accounts.accounts_utils.grp')
  def testGetGroup(self, mock_grp):
    mock_grp.getgrnam.return_value = 'Test'
    self.assertEqual(
        accounts_utils.AccountsUtils._GetGroup(self.mock_utils, 'valid'),
        'Test')
    mock_grp.getgrnam.side_effect = KeyError('Test Error')
    self.assertEqual(
        accounts_utils.AccountsUtils._GetGroup(self.mock_utils, 'invalid'),
        None)
    expected_calls = [
        mock.call.getgrnam('valid'),
        mock.call.getgrnam('invalid'),
    ]
    self.assertEqual(mock_grp.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testCreateSudoersGroup(self, mock_exists, mock_call, mock_permissions):
    mock_open = mock.mock_open()
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(mock_permissions, 'permissions')
    mocks.attach_mock(self.mock_utils._GetGroup, 'group')
    mocks.attach_mock(self.mock_logger, 'logger')
    self.mock_utils._GetGroup.return_value = False
    mock_exists.return_value = False
    command = self.groupadd_cmd.format(group=self.sudoers_group)

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      accounts_utils.AccountsUtils._CreateSudoersGroup(self.mock_utils)
      mock_open().write.assert_called_once_with(mock.ANY)

    expected_calls = [
        mock.call.group(self.sudoers_group),
        mock.call.call(command.split(' ')),
        mock.call.exists(self.sudoers_file),
        mock.call.permissions(self.sudoers_file, mode=0o440, uid=0, gid=0),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testCreateSudoersGroupSkip(
      self, mock_exists, mock_call, mock_permissions):
    mock_open = mock.mock_open()
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(mock_permissions, 'permissions')
    mocks.attach_mock(self.mock_utils._GetGroup, 'group')
    mocks.attach_mock(self.mock_logger, 'logger')
    self.mock_utils._GetGroup.return_value = True
    mock_exists.return_value = True

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      accounts_utils.AccountsUtils._CreateSudoersGroup(self.mock_utils)
      mock_open().write.assert_not_called()

    expected_calls = [
        mock.call.group(self.sudoers_group),
        mock.call.exists(self.sudoers_file),
        mock.call.permissions(self.sudoers_file, mode=0o440, uid=0, gid=0),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testCreateSudoersGroupError(
      self, mock_exists, mock_call, mock_permissions):
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(mock_call, 'call')
    mocks.attach_mock(mock_permissions, 'permissions')
    mocks.attach_mock(self.mock_utils._GetGroup, 'group')
    mocks.attach_mock(self.mock_logger, 'logger')
    self.mock_utils._GetGroup.return_value = False
    mock_exists.return_value = True
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')
    command = self.groupadd_cmd.format(group=self.sudoers_group)

    accounts_utils.AccountsUtils._CreateSudoersGroup(self.mock_utils)
    expected_calls = [
        mock.call.group(self.sudoers_group),
        mock.call.call(command.split(' ')),
        mock.call.logger.warning(mock.ANY, mock.ANY),
        mock.call.exists(self.sudoers_file),
        mock.call.permissions(self.sudoers_file, mode=0o440, uid=0, gid=0),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testCreateSudoersGroupWriteError(self, mock_exists):
    mock_open = mock.mock_open()
    mocks = mock.Mock()
    mocks.attach_mock(mock_exists, 'exists')
    mocks.attach_mock(self.mock_utils._GetGroup, 'group')
    mocks.attach_mock(self.mock_logger, 'logger')
    self.mock_utils._GetGroup.return_value = True
    mock_exists.return_value = False
    mock_open.side_effect = IOError()

    accounts_utils.AccountsUtils._CreateSudoersGroup(self.mock_utils)
    expected_calls = [
        mock.call.group(self.sudoers_group),
        mock.call.exists(self.sudoers_file),
        mock.call.logger.error(mock.ANY, self.sudoers_file, mock.ANY),
    ]
    self.assertEqual(mocks.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.pwd')
  def testGetUser(self, mock_pwd):
    mock_pwd.getpwnam.return_value = 'Test'
    self.assertEqual(
        accounts_utils.AccountsUtils._GetUser(self.mock_utils, 'valid'),
        'Test')
    mock_pwd.getpwnam.side_effect = KeyError('Test Error')
    self.assertEqual(
        accounts_utils.AccountsUtils._GetUser(self.mock_utils, 'invalid'),
        None)
    expected_calls = [
        mock.call.getpwnam('valid'),
        mock.call.getpwnam('invalid'),
    ]
    self.assertEqual(mock_pwd.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testAddUser(self, mock_call):
    user = 'user'
    command = self.useradd_cmd.format(user=user)

    self.assertTrue(
        accounts_utils.AccountsUtils._AddUser(self.mock_utils, user))
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [mock.call.info(mock.ANY, user)] * 2
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testAddUserError(self, mock_call):
    user = 'user'
    command = self.useradd_cmd.format(user=user)
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    self.assertFalse(
        accounts_utils.AccountsUtils._AddUser(self.mock_utils, user))
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [
        mock.call.info(mock.ANY, user),
        mock.call.warning(mock.ANY, user, mock.ANY),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testUpdateUserGroups(self, mock_call):
    user = 'user'
    groups = ['a', 'b', 'c']
    groups_string = ','.join(groups)
    command = self.usermod_cmd.format(user=user, groups=groups_string)

    self.assertTrue(
        accounts_utils.AccountsUtils._UpdateUserGroups(
            self.mock_utils, user, groups))
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [
        mock.call.debug(mock.ANY, user, groups_string),
        mock.call.debug(mock.ANY, user),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testUpdateUserGroupsError(self, mock_call):
    user = 'user'
    groups = ['a', 'b', 'c']
    groups_string = ','.join(groups)
    command = self.usermod_cmd.format(user=user, groups=groups_string)
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    self.assertFalse(
        accounts_utils.AccountsUtils._UpdateUserGroups(
            self.mock_utils, user, groups))
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [
        mock.call.debug(mock.ANY, user, groups_string),
        mock.call.warning(mock.ANY, user, mock.ANY),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.shutil.copy')
  @mock.patch('google_compute_engine.accounts.accounts_utils.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.islink')
  def testUpdateAuthorizedKeys(
      self, mock_islink, mock_exists, mock_tempfile, mock_copy,
      mock_permissions):
    mock_open = mock.mock_open()
    user = 'user'
    ssh_keys = ['Google key 1', 'Google key 2']
    temp_dest = '/tmp/dest'
    pw_uid = 1
    pw_gid = 2
    pw_dir = '/home'
    ssh_dir = '/home/.ssh'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', pw_uid, pw_gid, '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_islink.return_value = False
    mock_exists.return_value = True
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.__enter__.return_value.name = temp_dest
    self.mock_logger.name = 'test'

    with mock.patch('%s.open' % builtin, mock_open, create=False):
      mock_open().readlines.return_value = [
          'User key a\n',
          'User key b\n',
          '\n',
          self.mock_utils.google_comment + '\n',
          'Google key a\n',
          self.mock_utils.google_comment + '\n',
          'Google key b\n',
          'User key c\n',
      ]
      accounts_utils.AccountsUtils._UpdateAuthorizedKeys(
          self.mock_utils, user, ssh_keys)

    expected_calls = [
        mock.call(mode='w', prefix='test-', delete=True),
        mock.call.__enter__(),
        mock.call.__enter__().write('User key a\n'),
        mock.call.__enter__().write('User key b\n'),
        mock.call.__enter__().write('\n'),
        mock.call.__enter__().write('User key c\n'),
        mock.call.__enter__().write(self.mock_utils.google_comment + '\n'),
        mock.call.__enter__().write('Google key 1\n'),
        mock.call.__enter__().write(self.mock_utils.google_comment + '\n'),
        mock.call.__enter__().write('Google key 2\n'),
        mock.call.__enter__().flush(),
        mock.call.__exit__(None, None, None),
    ]
    self.assertEqual(mock_tempfile.mock_calls, expected_calls)
    mock_copy.assert_called_once_with(temp_dest, authorized_keys_file)
    expected_calls = [
        mock.call(ssh_dir, mode=0o700, uid=pw_uid, gid=pw_gid, mkdir=True),
        mock.call(authorized_keys_file, mode=0o600, uid=pw_uid, gid=pw_gid),
    ]
    self.assertEqual(mock_permissions.mock_calls, expected_calls)
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.shutil.copy')
  @mock.patch('google_compute_engine.accounts.accounts_utils.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.islink')
  def testUpdateAuthorizedKeysNoKeys(
      self, mock_islink, mock_exists, mock_tempfile, mock_copy,
      mock_permissions):
    user = 'user'
    ssh_keys = ['Google key 1']
    temp_dest = '/tmp/dest'
    pw_uid = 1
    pw_gid = 2
    pw_dir = '/home'
    ssh_dir = '/home/.ssh'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', pw_uid, pw_gid, '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_islink.return_value = False
    mock_exists.side_effect = [True, False]
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.__enter__.return_value.name = temp_dest
    self.mock_logger.name = 'test'

    # The authorized keys file does not exist so write a new one.
    accounts_utils.AccountsUtils._UpdateAuthorizedKeys(
        self.mock_utils, user, ssh_keys)
    expected_calls = [
        mock.call(mode='w', prefix='test-', delete=True),
        mock.call.__enter__(),
        mock.call.__enter__().write(self.mock_utils.google_comment + '\n'),
        mock.call.__enter__().write('Google key 1\n'),
        mock.call.__enter__().flush(),
        mock.call.__exit__(None, None, None),
    ]
    self.assertEqual(mock_tempfile.mock_calls, expected_calls)
    mock_copy.assert_called_once_with(temp_dest, authorized_keys_file)
    expected_calls = [
        mock.call(ssh_dir, mode=0o700, uid=pw_uid, gid=pw_gid, mkdir=True),
        mock.call(authorized_keys_file, mode=0o600, uid=pw_uid, gid=pw_gid),
    ]
    self.assertEqual(mock_permissions.mock_calls, expected_calls)
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.shutil.copy')
  @mock.patch('google_compute_engine.accounts.accounts_utils.tempfile.NamedTemporaryFile')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.islink')
  def testUpdateAuthorizedKeysNoHomeDir(
      self, mock_islink, mock_exists, mock_tempfile, mock_copy,
      mock_permissions):
    user = 'user'
    ssh_keys = ['Google key 1']
    temp_dest = '/tmp/dest'
    pw_uid = 1
    pw_gid = 2
    pw_dir = '/home/user'
    ssh_dir = '/home/user/.ssh'
    authorized_keys_file = '/home/user/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', pw_uid, pw_gid, '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_islink.return_value = False
    mock_exists.side_effect = [False, False]
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.__enter__.return_value.name = temp_dest
    self.mock_logger.name = 'test'
    accounts_utils.AccountsUtils._UpdateAuthorizedKeys(
        self.mock_utils, user, ssh_keys)
    expected_calls = [
        mock.call(pw_dir, mode=0o755, uid=pw_uid, gid=pw_gid, mkdir=True),
        mock.call(ssh_dir, mode=0o700, uid=pw_uid, gid=pw_gid, mkdir=True),
        mock.call(authorized_keys_file, mode=0o600, uid=pw_uid, gid=pw_gid),
    ]
    self.assertEqual(mock_permissions.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  def testUpdateAuthorizedKeysNoUser(self, mock_permissions):
    user = 'user'
    ssh_keys = ['key']
    self.mock_utils._GetUser.return_value = None

    # The user does not exist, so do not write authorized keys.
    accounts_utils.AccountsUtils._UpdateAuthorizedKeys(
        self.mock_utils, user, ssh_keys)
    self.mock_utils._GetUser.assert_called_once_with(user)
    mock_permissions.assert_not_called()
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.islink')
  def testUpdateAuthorizedKeysSymlink(self, mock_islink, mock_permissions):
    user = 'user'
    ssh_keys = ['Google key 1']
    pw_uid = 1
    pw_gid = 2
    pw_dir = '/home'
    ssh_dir = '/home/.ssh'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', pw_uid, pw_gid, '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_islink.side_effect = [False, True]

    accounts_utils.AccountsUtils._UpdateAuthorizedKeys(
        self.mock_utils, user, ssh_keys)
    expected_calls = [mock.call(ssh_dir), mock.call(authorized_keys_file)]
    self.assertEqual(mock_islink.mock_calls, expected_calls)
    self.mock_logger.warning.assert_called_once_with(mock.ANY, user)
    mock_permissions.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testUpdateSudoer(self, mock_call):
    user = 'user'
    command = self.gpasswd_remove_cmd.format(
        user=user, group=self.sudoers_group)

    self.assertTrue(
        accounts_utils.AccountsUtils._UpdateSudoer(self.mock_utils, user))
    mock.call.assert_called_once_with(command.split(' '))
    expected_calls = [
        mock.call.info(mock.ANY, user),
        mock.call.debug(mock.ANY, user),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testUpdateSudoerAddSudoer(self, mock_call):
    user = 'user'
    command = self.gpasswd_add_cmd.format(user=user, group=self.sudoers_group)

    self.assertTrue(
        accounts_utils.AccountsUtils._UpdateSudoer(
            self.mock_utils, user, sudoer=True))
    mock.call.assert_called_once_with(command.split(' '))
    expected_calls = [
        mock.call.info(mock.ANY, user),
        mock.call.debug(mock.ANY, user),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testUpdateSudoerError(self, mock_call):
    user = 'user'
    command = self.usermod_cmd.format(user=user, groups=self.sudoers_group)
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')

    self.assertFalse(
        accounts_utils.AccountsUtils._UpdateSudoer(self.mock_utils, user))
    mock.call.assert_called_once_with(command.split(' '))
    expected_calls = [
        mock.call.info(mock.ANY, user),
        mock.call.warning(mock.ANY, user, mock.ANY),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testRemoveAuthorizedKeys(self, mock_exists, mock_remove):
    user = 'user'
    pw_dir = '/home'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', '', '', '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_exists.return_value = True

    accounts_utils.AccountsUtils._RemoveAuthorizedKeys(self.mock_utils, user)
    self.mock_utils._GetUser.assert_called_once_with(user)
    mock_exists.assert_called_once_with(authorized_keys_file)
    mock_remove.assert_called_once_with(authorized_keys_file)
    self.mock_logger.warning.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testRemoveAuthorizedKeysNoKeys(self, mock_exists, mock_remove):
    user = 'user'
    pw_dir = '/home'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', '', '', '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_exists.return_value = False

    accounts_utils.AccountsUtils._RemoveAuthorizedKeys(self.mock_utils, user)
    self.mock_utils._GetUser.assert_called_once_with(user)
    mock_exists.assert_called_once_with(authorized_keys_file)
    mock_remove.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testRemoveAuthorizedKeysNoUser(self, mock_exists):
    user = 'user'
    self.mock_utils._GetUser.return_value = None

    accounts_utils.AccountsUtils._RemoveAuthorizedKeys(self.mock_utils, user)
    self.mock_utils._GetUser.assert_called_once_with(user)
    mock_exists.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.remove')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testRemoveAuthorizedKeysError(self, mock_exists, mock_remove):
    user = 'user'
    pw_dir = '/home'
    authorized_keys_file = '/home/.ssh/authorized_keys'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', '', '', '', pw_dir, ''))
    self.mock_utils._GetUser.return_value = pw_entry
    mock_exists.return_value = True
    mock_remove.side_effect = OSError('Test Error')

    accounts_utils.AccountsUtils._RemoveAuthorizedKeys(self.mock_utils, user)
    self.mock_utils._GetUser.assert_called_once_with(user)
    mock_exists.assert_called_once_with(authorized_keys_file)
    mock_remove.assert_called_once_with(authorized_keys_file)
    self.mock_logger.warning.assert_called_once_with(mock.ANY, user, mock.ANY)

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testGetConfiguredUsers(self, mock_exists):
    mock_open = mock.mock_open()
    mock_exists.return_value = True
    with mock.patch('%s.open' % builtin, mock_open, create=False):
      mock_open().readlines.return_value = ['a\n', 'b\n', 'c\n', '\n']
      self.assertEqual(
          accounts_utils.AccountsUtils.GetConfiguredUsers(self.mock_utils),
          ['a', 'b', 'c', ''])

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  def testGetConfiguredUsersEmpty(self, mock_exists):
    mock_exists.return_value = False
    self.assertEqual(
        accounts_utils.AccountsUtils.GetConfiguredUsers(self.mock_utils), [])

  @mock.patch('google_compute_engine.accounts.accounts_utils.os.makedirs')
  @mock.patch('google_compute_engine.accounts.accounts_utils.os.path.exists')
  @mock.patch('google_compute_engine.accounts.accounts_utils.file_utils.SetPermissions')
  @mock.patch('google_compute_engine.accounts.accounts_utils.shutil.copy')
  @mock.patch('google_compute_engine.accounts.accounts_utils.tempfile.NamedTemporaryFile')
  def testSetConfiguredUsers(
      self, mock_tempfile, mock_copy, mock_permissions, mock_exists,
      mock_makedirs):
    temp_dest = '/temp/dest'
    users = ['a', 'b', 'c']
    mock_tempfile.return_value = mock_tempfile
    mock_tempfile.__enter__.return_value.name = temp_dest
    mock_exists.return_value = False
    self.mock_logger.name = 'test'

    accounts_utils.AccountsUtils.SetConfiguredUsers(self.mock_utils, users)

    expected_calls = [
        mock.call(mode='w', prefix='test-', delete=True),
        mock.call.__enter__(),
        mock.call.__enter__().write('a\n'),
        mock.call.__enter__().write('b\n'),
        mock.call.__enter__().write('c\n'),
        mock.call.__enter__().flush(),
        mock.call.__exit__(None, None, None),
    ]
    self.assertEqual(mock_tempfile.mock_calls, expected_calls)
    mock_makedirs.assert_called_once_with(self.users_dir)
    mock_copy.assert_called_once_with(temp_dest, self.users_file)
    mock_permissions.assert_called_once_with(
        self.users_file, mode=0o600, uid=0, gid=0)

  def testUpdateUser(self):
    valid_users = [
        'user',
        '_',
        '.',
        '.abc_',
        '_abc-',
        'ABC',
        'A_.-',
    ]
    groups = ['a', 'b', 'c']
    keys = ['Key 1', 'Key 2']
    pw_entry = accounts_utils.pwd.struct_passwd(tuple(['']*7))
    self.mock_utils.groups = groups
    self.mock_utils._GetUser.return_value = pw_entry
    self.mock_utils._AddUser.return_value = True
    self.mock_utils._UpdateUserGroups.return_value = True
    for user in valid_users:
      self.assertTrue(
          accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, keys))
      self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=True)
      self.mock_utils._UpdateSudoer.reset_mock()
      self.mock_utils._UpdateAuthorizedKeys.assert_called_once_with(user, keys)
      self.mock_utils._UpdateAuthorizedKeys.reset_mock()
    self.mock_logger.warning.assert_not_called()

  def testUpdateUserInvalidUser(self):
    self.mock_utils._GetUser = mock.Mock()
    invalid_users = [
        '',
        '!#$%^',
        '-abc',
        '#abc',
        '^abc',
        'abc*xyz',
        'abc xyz',
        'xyz*',
        'xyz$',
        'areallylongusernamethatexceedsthethirtytwocharacterlimit'
    ]
    for user in invalid_users:
      self.assertFalse(
          accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, []))
      self.mock_logger.warning.assert_called_once_with(mock.ANY, user)
      self.mock_logger.reset_mock()
    self.mock_utils._GetUser.assert_not_called()

  def testUpdateUserFailedAddUser(self):
    self.mock_utils._UpdateUserGroups = mock.Mock()
    user = 'user'
    self.mock_utils._GetUser.return_value = False
    self.mock_utils._AddUser.return_value = False

    self.assertFalse(
        accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, []))
    self.mock_utils._GetUser.assert_called_once_with(user)
    self.mock_utils._AddUser.assert_called_once_with(user)

  def testUpdateUserFailedUpdateGroups(self):
    user = 'user'
    groups = ['a', 'b', 'c']
    self.mock_utils.groups = groups
    self.mock_utils._GetUser.return_value = False
    self.mock_utils._AddUser.return_value = True
    self.mock_utils._UpdateUserGroups.return_value = False

    self.assertFalse(
        accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, []))
    self.mock_utils._GetUser.assert_called_once_with(user)
    self.mock_utils._AddUser.assert_called_once_with(user)
    self.mock_utils._UpdateUserGroups.assert_called_once_with(user, groups)

  def testUpdateUserNoLogin(self):
    self.mock_utils._UpdateAuthorizedKeys = mock.Mock()
    user = 'user'
    groups = ['a', 'b', 'c']
    pw_shell = '/sbin/nologin'
    pw_entry = accounts_utils.pwd.struct_passwd(
        ('', '', '', '', '', '', pw_shell))
    self.mock_utils.groups = groups
    self.mock_utils._GetUser.return_value = pw_entry
    self.mock_utils._UpdateUserGroups.return_value = True

    self.assertTrue(
        accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, []))
    self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=True)
    self.mock_utils._UpdateAuthorizedKeys.assert_not_called()

  def testUpdateUserSudoersError(self):
    user = 'user'
    groups = ['a', 'b', 'c']
    keys = ['Key 1', 'Key 2']
    pw_entry = accounts_utils.pwd.struct_passwd(tuple(['']*7))
    self.mock_utils.groups = groups
    self.mock_utils._GetUser.return_value = pw_entry
    self.mock_utils._UpdateSudoer.return_value = False

    self.assertFalse(
        accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, keys))
    self.mock_utils._GetUser.assert_called_once_with(user)
    self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=True)

  def testUpdateUserError(self):
    user = 'user'
    groups = ['a', 'b', 'c']
    keys = ['Key 1', 'Key 2']
    pw_entry = accounts_utils.pwd.struct_passwd(tuple(['']*7))
    self.mock_utils.groups = groups
    self.mock_utils._GetUser.return_value = pw_entry
    self.mock_utils._AddUser.return_value = True
    self.mock_utils._UpdateAuthorizedKeys.side_effect = IOError('Test Error')

    self.assertFalse(
        accounts_utils.AccountsUtils.UpdateUser(self.mock_utils, user, keys))
    self.mock_logger.warning.assert_called_once_with(mock.ANY, user, mock.ANY)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testRemoveUser(self, mock_call):
    user = 'user'
    self.mock_utils.remove = False

    accounts_utils.AccountsUtils.RemoveUser(self.mock_utils, user)
    self.mock_utils._RemoveAuthorizedKeys.assert_called_once_with(user)
    self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=False)
    mock_call.assert_not_called()

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testRemoveUserForce(self, mock_call):
    user = 'user'
    command = self.userdel_cmd.format(user=user)
    self.mock_utils.remove = True

    accounts_utils.AccountsUtils.RemoveUser(self.mock_utils, user)
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [mock.call.info(mock.ANY, user)] * 2
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)
    self.mock_utils._RemoveAuthorizedKeys.assert_called_once_with(user)
    self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=False)

  @mock.patch('google_compute_engine.accounts.accounts_utils.subprocess.check_call')
  def testRemoveUserError(self, mock_call):
    user = 'user'
    command = self.userdel_cmd.format(user=user)
    mock_call.side_effect = subprocess.CalledProcessError(1, 'Test')
    self.mock_utils.remove = True

    accounts_utils.AccountsUtils.RemoveUser(self.mock_utils, user)
    mock.call.assert_called_once_with(command.split(' ')),
    expected_calls = [
        mock.call.info(mock.ANY, user),
        mock.call.warning(mock.ANY, user, mock.ANY),
    ]
    self.assertEqual(self.mock_logger.mock_calls, expected_calls)
    self.mock_utils._RemoveAuthorizedKeys.assert_called_once_with(user)
    self.mock_utils._UpdateSudoer.assert_called_once_with(user, sudoer=False)


if __name__ == '__main__':
  unittest.main()
